--[========================================================================[--

Menu screen code for Thrust II Reloaded.

Copyright © 2015-2018 Pedro Gimeno Fortea

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

--]========================================================================]--

local menu = {}

local la,le,lfs,lf,lg,li,lj,lk,lm,lmo,lp,ls,lsys,lth,lt,lw = require'ns'()

local menuimage
local has_save
local t
local playing
local period = 1.9961678004535148 -- 88031 samples / 44100 Hz
local bar1, bar2
local barc
local snd
local activeMenu
local sndstate = "stopped"

local DIV = love_version >= 11000000 and 255 or 1

function menu.load()
  menuimage = lg.newImage("img/mainmenu.png")
--    playground~/images/ThrustII-newlogo-isolated.png")
  snd = la.newSource("snd/Thrust II - menu.ogg", "stream")
  snd:setVolume(0.5)
  snd:setLooping(true)
end

function menu.activate()
	
        thrustsnd:stop()

  has_save = main.isFile("saved.txt")
  playing = false
  activeMenu = "main"
end

function menu.deactivate()
  snd:stop()
  sndstate = "stopped"
end

function menu.pause(paused)
  if playing then
    if paused then
      if sndstate == "playing" then
        snd:pause()
        sndstate = "paused"
      end
    elseif sndstate == "paused" then
      snd:play()
      sndstate = "playing"
    end
  end
end

function menu.update(dt)

if activeMenu == "main" then
		
    if button1pressed==true or button2pressed==true or button3pressed==true or button4pressed==true  then
      main.restore = false
      screens.game.newgame()
      screens.getready.fromstart = true
      main.activate(screens.getready)
      return
    end
end

 -- Set shader uniforms for primary light source
        vectorNormalsShader:send("LightPos1", {0.5,0.5,0.1})
        vectorNormalsShader:send("LightColor1", {1,1,1,0.5})
        vectorNormalsShader:send("AmbientColor", {1,0.5,0.5,1.0})
        
         -- Set other shader parameters for normal mapping
        vectorNormalsShader:send("Resolution", {love.graphics.getWidth(), love.graphics.getHeight()})
        vectorNormalsShader:send("Falloff", {0.2, 0.8, 1.2})
  -- This is obsolete and was used for synchronization of the audio effect with the bars.
  if not playing then
    -- start playing (not done earlier because we don't know how long it would take)
    --playing = true
    t = period/2
    if main.music then
      --snd:play()
      --sndstate = "playing"
    end
    bar1 = main.wcy
    bar2 = bar1
    barc = 1
  else
    t = t + dt
    if t >= period then
      t = t - period
    end
    bar1 = main.wcy - (t-period/2)/period * main.wh*0.78
    bar2 = main.wcy + (t-period/2)/period * main.wh*0.78
    barc = barc + dt * 50
  end
end

function menu.draw()
  if playing then
    local c = math.floor(barc) % 6 + 1
    local b = c % 2  c = (c - b) / 2
    local r = c % 2  c = (c - r) / 2
    local g = c % 2
    lg.setColor(r*(192/DIV), g*(192/DIV), b*(192/DIV))
    lg.rectangle("fill", 0, bar1-2, main.ww, 5)
    lg.rectangle("fill", 0, bar2-2, main.ww, 5)
  end
  lg.setColor(255/DIV,255/DIV,255/DIV)
  main.drawaligned(5, menuimage, main.wcx, main.wcy)
--  screens.game.tiles_draw(2176, 32)
--  do return end

  lg.setColor(0,192/DIV,0)
  main.centertext("@^ A.ROGERS 1986", main.wcx, main.wcy)
  main.centertext("@^ P.GIMENO 2017", main.wcx, main.wcy+24)
  lg.setColor(192/DIV,192/DIV,192/DIV)
  if activeMenu == "main" then
    main.centertext("1....START  GAME", main.wcx, main.wcy+32*3)
    main.centertext("2....DEFINE KEYS", main.wcx, main.wcy+32*4)
    main.centertext("3....SET OPTIONS", main.wcx, main.wcy+32*5)
    if has_save then
      main.centertext("4....LOAD   GAME", main.wcx, main.wcy+32*6)
    end
        main.centertext("5....TOGGLE TOUCH", main.wcx, main.wcy+32*7)
        main.centertext("6....ABOUT", main.wcx, main.wcy+32*8)
  elseif activeMenu == "options" then
    local musOnOff
    if main.music then
      musOnOff = "-  ON"
    else
      musOnOff = "- OFF"
    end
    main.centertext(string.format("1....ZOOM - %3.2gX", main.pixelsize), main.wcx, main.wcy+32*3)
    main.centertext(main.fullscreen and "2....WINDOW MODE" or "2....FULL SCREEN", main.wcx, main.wcy+32*4)
    main.centertext("3....MUSIC "..musOnOff, main.wcx, main.wcy+32*5)
    main.centertext("0....MAIN   MENU", main.wcx, main.wcy+32*7)
   elseif activeMenu == "about" then
    main.centertext("0....MAIN   MENU", main.wcx, main.wcy+32*3)
	--main.centertext("TEST", main.wcx, main.wcy+32*5)
	
	love.graphics.setShader()
	love.graphics.setColor(0,0,0,0.8)
	love.graphics.rectangle ("fill",main.wcx/10, main.wcy/10,main.ww,main.wh)
	love.graphics.setColor(1,1,1,1)
	love.graphics.setShader(vectorNormalsShader)
	printCredits(main.wcx/10, main.wcy/10)
   
  end
  lg.setColor(255/DIV,255/DIV,255/DIV)
end

-- Table to hold all the credits text
aboutText = {
  "Original game: ",
  "  Copyright © 1986 Andrew Rogers.",
  "",
  "Code for this version:",
  "  Copyright © 2015-2018 Pedro Gimeno Fortea. License: Expat.",
  "  Each individual file includes the full license text.",
  "",
  "Touch controls, new assets, new shaders and other enhancements:",
  "  Glitchapp 2024",
  "",
  "Third party libraries:",
  "- 3rdparty/dkjson.lua:",
  "  Copyright (C) 2010-2013 David Heiko Kolf. License: Expat.",
  "  See the file for the full license text.",
  "",
  "Graphics:",
  "- img/splash.png:",
  "  By J.C.G. and Cucurbitacée. Version by Curcubitacée used with permission.",
  "",
  "- img/Explosion32x32.png:",
  "  By Curcubitacée. Used with permission.",
  "",
  "- img/Shot.png:",
  "  By Pedro Gimeno Fortea, donated to the public domain.",
  "",
  "Sound effects:",
  "- snd/latch.wav:",
  "  By an anonymous contributor. Used with permission.",
  "",
  "- snd/Grenade-SoundBible.com-1777900486.ogg:",
  "  By Mike Koenig, licensed under [CC-BY 3.0](https://creativecommons.org/licenses/by/3.0/)",
  "  File also available at: http://soundbible.com/1151-Grenade.html",
  "  Compressed from WAV to OGG with oggenc by Pedro Gimeno, to work around bug",
  "  in LÖVE 0.9.2.",
  "",
  "- snd/LoadAgent.wav:",
  "  Original by Mike Koenig, licensed under [CC-BY 3.0](https://creativecommons.org/licenses/by/3.0/).",
  "  Modifications Copyright © 2017 Pedro Gimeno Fortea.",
  "  Modified version licensed under CC-BY 3.0 as well.",
  "  Original file available at: http://soundbible.com/845-Suction.html",
  "",
  "- snd/EndGame.ogg, snd/orbdanger.wav:",
  "  Copyright © 2016 Pedro Gimeno Fortea",
  "  Licensed under [WTFPL](http://www.wtfpl.net/txt/copying/) version 2",
  "  as published by Sam Hocevar. Full license text follows (note that the",
  "  copyright statement within the license applies to the license text itself,",
  "  not to the work):",
  "",
  "              DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE",
  "                       Version 2, December 2004",
  "",
  "  Copyright (C) 2004 Sam Hocevar <sam@hocevar.net>",
  "",
  "  Everyone is permitted to copy and distribute verbatim or modified",
  "  copies of this license document, and changing it is allowed as long",
  "  as the name is changed.",
  "",
  "             DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE",
  "    TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION",
  "",
  "   0. You just DO WHAT THE FUCK YOU WANT TO.",
  "",
  "- snd/Cable_Sound.wav: Original from the public domain sfx corresponding to",
  "  the game Golgotha, by Crack dot com, who donated the assets to the public",
  "  domain. Modified for use with this game; the modifications are copyright ©",
  "  2017 Pedro Gimeno Fortea, licensed under the WTFPL (see license text above).",
  "",
  "- snd/supertank_plazma_fireball_22khz.mp3:",
  "  Public domain sfx originally from the game Golgotha, by Crack dot com, who",
  "  donated the assets to the public domain.",
  "",
  "- snd/AgentShot.wav, snd/click_one_22khz.wav, snd/click_two_22khz.wav,",
  "  snd/jet_lp.ogg:",
  "  Public domain sfx originally from the game Golgotha, by Crack dot com, who",
  "  donated the assets to the public domain. Modifications Copyright © 2017",
  "  Pedro Gimeno Fortea. Modified version licensed under WTFPL version 2.",
  "  AgentShot.wav comes from acid.mp3; click_*_22khz.wav come from the",
  "  corresponding mp3, and jet_lp.ogg comes from jet_lp.wav.",
  "",
  "Music:",
  "- snd/Thrust II - menu.ogg:",
  "  Composed and performed by Javier Fopiani.",
  "  Licensed under [CC-BY-SA 4.0](http://creativecommons.org/licenses/by-sa/4.0/)",
  "  (Attribution, share-alike).",
  "",
  "- snd/Thrust II - in game - Loop 909924-6942804.ogg:",
  "  Original version composed and performed by Javier Fopiani.",
  "  Modifications to the original by Pedro Gimeno to enable smooth looping.",
  "  Licensed under [CC-BY-SA 4.0](http://creativecommons.org/licenses/by-sa/4.0/)",
  "  (Attribution, share-alike).",
  "",
  "- snd/d_runnin-looped_862870-10353420.ogg:",
  "  Copyright (c) 2001-2003 Contributors to the Freedoom project.",
  "  All rights reserved.",
  "",
  "  Redistribution and use in source and binary forms, with or without",
  "  modification, are permitted provided that the following conditions",
  "  are met:",
  "",
  "  * Redistributions of source code must retain the above copyright",
  "    notice, this list of conditions and the following disclaimer.",
  "  * Redistributions in binary form must reproduce the above copyright",
  "    notice, this list of conditions and the following disclaimer",
  "    in the documentation and/or other materials provided with the",
  "    distribution.",
  "  * Neither the name of the freedoom project nor the names of its",
  "    contributors may be used to endorse or promote products derived",
  "    from this software without specific prior written permission.",
  "",
  "  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS",
  "  \"AS IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT",
  "  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS",
  "  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE",
  "  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,",
  "  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,",
  "  https://freedoom.github.io/about.html"
}

-- Function to print the credits
function printCredits(x, y)
  local lineHeight = love.graphics.getFont():getHeight() / 0.5
  local maxWidth = 1000
  local currentY = y

  for _, line in ipairs(aboutText) do
    love.graphics.printf(line, x, currentY, maxWidth, "left")
    currentY = currentY + lineHeight
  end
end

function menu.keypressed(k)
  if k == "escape" then
    if activeMenu == "options" then
      activeMenu = "main"
    else
      le.quit()
    end
  end
end

function menu.textinput(c)
  if activeMenu == "main" then
    if c == "1" then
      main.restore = false
      screens.game.newgame()
      screens.getready.fromstart = true
      main.activate(screens.getready)
      return
    elseif c == "2" then
      main.activate(screens.redef)
      return
    elseif c == "3" then
      activeMenu = "options"
      return
    elseif c == "4" and has_save then
      main.restore = true
      screens.game.newgame()
      screens.getready.fromstart = true
      return main.activate(screens.getready)
    elseif c == "5" then
		touchEnabled = not touchEnabled
	elseif c == "6" then
		activeMenu = "about"
		
    end
  elseif activeMenu == "options" then
    if c == "0" then
      activeMenu = "main"
    elseif c == "1" then
      main.pixelsize = main.pixelsize + 0.5
      if main.pixelsize > 4 then
        main.pixelsize = 1
      end
      love.resize(lg.getDimensions())
      main.saveScreen()
    elseif c == "2" then
      main.fullscreen = not main.fullscreen
      main.setMode()
      main.saveScreen()
      -- HACK: Apparently LÖVE neglects to call love.resize after entering/exiting fullscreen.
      love.resize(lg.getDimensions())
    elseif c == "3" then
      main.music = not main.music
      if main.music then
        snd:play()
        sndstate = "playing"
      else
        snd:stop()
        sndstate = "stopped"
      end
      main.setVolume(main.vol, true) -- save music setting
    end
   elseif activeMenu == "about" then
    if c == "0" then
      activeMenu = "main"
   end
  end
end

return menu
